#include <QInputDialog>
#include <QMessageBox>

#include "sendbuttonmenu.h"
#include "psioptions.h"
#include "psiiconset.h"

SendButtonTemplatesEditor::SendButtonTemplatesEditor(QWidget* parent)
	: QDialog(parent)
{
	ui_.setupUi(this);
	ui_.lstTemplates->addItems(PsiOptions::instance()->getOption("options.ui.chat.templates", QStringList()).toStringList());
	setAttribute(Qt::WA_DeleteOnClose);
	editAction = new QAction(IconsetFactory::icon("psi/action_templates_edit").icon(), tr("Edit"), this);
	removeAction = new QAction(IconsetFactory::icon("psi/remove").icon(), tr("Remove"), this);
	upAction = new QAction(IconsetFactory::icon("psi/arrowUp").icon(), tr("Up"), this);
	downAction = new QAction(IconsetFactory::icon("psi/arrowDown").icon(), tr("Down"), this);
}

void SendButtonTemplatesEditor::addTemplate()
{
	QString t = ui_.txtTemplate->text().trimmed();
	if (t.isEmpty() || ui_.lstTemplates->count() >= 50) // 50 - default limit templates
		return;
	ui_.lstTemplates->addItem(t);
	ui_.txtTemplate->setText("");
}

void SendButtonTemplatesEditor::removeTemplate()
{
	int idx=ui_.lstTemplates->currentRow();
	if (idx>=0) {
		QListWidgetItem *item =ui_.lstTemplates->takeItem(idx);
		if (item) delete item;
	}
}

void SendButtonTemplatesEditor::editTemplate()
{
	int idx = ui_.lstTemplates->currentRow();
	if (idx < 0)
		return;
	QString templ_str = ui_.lstTemplates->currentItem()->text();
	bool fOk = false;
	QString new_templ_str = QInputDialog::getText(this, QString::fromUtf8("Edit template"), QString::fromUtf8("Input new template text"), QLineEdit::Normal, templ_str, &fOk);
	if (fOk && !new_templ_str.isEmpty() && templ_str != new_templ_str) {
		QListWidgetItem *item = ui_.lstTemplates->item(idx);
		if (item)
			item->setText(new_templ_str);
	}
}

void SendButtonTemplatesEditor::upTemplate()
{
	swapItem(-1);
}

void SendButtonTemplatesEditor::downTemplate()
{
	swapItem(1);
}

void SendButtonTemplatesEditor::selectionChanged()
{
	bool up_e = false;
	bool dw_e = false;
	bool ed_e = false;
	int idx = ui_.lstTemplates->currentRow();
	if (idx >= 0) {
		int cnt = ui_.lstTemplates->count();
		if (cnt > 0) {
			if (idx > 0)
				up_e = true;
			if (idx < cnt - 1)
				dw_e = true;
			ed_e = true;
		}
	}
	ui_.btnEdit->setEnabled(ed_e);
	editAction->setEnabled(ed_e);
	ui_.btnRemove->setEnabled(ed_e);
	removeAction->setEnabled(ed_e);
	ui_.btnUp->setEnabled(up_e);
	upAction->setEnabled(up_e);
	ui_.btnDown->setEnabled(dw_e);
	downAction->setEnabled(dw_e);
}

void SendButtonTemplatesEditor::contextMenu(QPoint pos)
{
	QMenu* menu = new QMenu(this);
	menu->addAction(editAction);
	menu->addAction(removeAction);
	menu->addSeparator();
	menu->addAction(upAction);
	menu->addAction(downAction);
	QAction *result = menu->exec(ui_.lstTemplates->mapToGlobal(pos));
	if (result == editAction) {
		editTemplate();
	} else if (result == removeAction) {
		removeTemplate();
	} else if (result == upAction) {
		upTemplate();
	} else if (result == downAction) {
		downTemplate();
	}
	delete menu;
}

void SendButtonTemplatesEditor::swapItem(int updown)
{
	int idx = ui_.lstTemplates->currentRow();
	if (idx >= 0) {
		int new_pos = idx + updown;
		if (new_pos >= 0 && new_pos < ui_.lstTemplates->count()) {
			QListWidgetItem *item = ui_.lstTemplates->takeItem(idx);
			if (item) {
				ui_.lstTemplates->insertItem(new_pos, item);
				ui_.lstTemplates->setCurrentItem(item);
			}
		}
	}
}

void SendButtonTemplatesEditor::accept()
{
	QString t = ui_.txtTemplate->text().trimmed();
	if (!t.isEmpty()) {
		QMessageBox *msgBox = new QMessageBox(this);
		msgBox->setIcon(QMessageBox::Warning);
		msgBox->setWindowTitle(tr("Save templates"));
		msgBox->setText(tr("The template \"%1\" hasn't been saved!").arg(t));
		msgBox->setInformativeText(tr("Continue?"));
		msgBox->setStandardButtons(QMessageBox::Yes | QMessageBox::No);
		msgBox->setDefaultButton(QMessageBox::No);
		msgBox->setWindowModality(Qt::WindowModal);
		int res = msgBox->exec();
		delete msgBox;
		if (res == QMessageBox::No) {
			return;
		}
	}
	QStringList templates;
	int cnt = ui_.lstTemplates->count();
	for (int i = 0; i < cnt; ++i)
		templates.append(ui_.lstTemplates->item(i)->text());
	PsiOptions::instance()->setOption("options.ui.chat.templates", templates);
	QDialog::accept();
}

//------------------------------

SendButtonTemplatesMenu::SendButtonTemplatesMenu(QWidget* parent)
	: QMenu(parent)
	, mapper(NULL)
	, ps_(false)
{
	setSeparatorsCollapsible(true);
	update();
	connect(PsiOptions::instance(), SIGNAL(optionChanged(const QString&)), SLOT(optionChanged(const QString&)));
}

void SendButtonTemplatesMenu::setParams(bool ps)
{
	if (ps_ != ps) {
		ps_ = ps;
		updatePsAction();
	}
}

void SendButtonTemplatesMenu::updatePsAction()
{
	bool ps_v = false;
	if (ps_ && !PsiOptions::instance()->getOption("options.ui.chat.disable-paste-send").toBool())
		ps_v = true;
	pasteSendAct->setVisible(ps_v);
}

void SendButtonTemplatesMenu::update()
{
	clear();
	if (mapper)
		delete mapper;
	mapper = new QSignalMapper(this);

	pasteSendAct = new QAction(IconsetFactory::icon("psi/action_paste_and_send").icon(), tr("Paste and &Send"), this);
	connect(pasteSendAct, SIGNAL(triggered()), SIGNAL(doPasteAndSend()));
	addAction(pasteSendAct);
	updatePsAction();
	addSeparator();

	QAction *editTemplatesAct = new QAction(IconsetFactory::icon("psi/action_templates_edit").icon(), tr("&Edit Templates"), this);
	connect(editTemplatesAct, SIGNAL(triggered()), SIGNAL(doEditTemplates()));
	addAction(editTemplatesAct);

	onlyPaste = new QAction(tr("Only &Paste"), this);
	onlyPaste->setCheckable(true);
	onlyPaste->setChecked(PsiOptions::instance()->getOption("options.ui.chat.only-paste-template").toBool());
	connect(onlyPaste, SIGNAL(triggered()), SLOT(clickOnlyPaste()));
	addAction(onlyPaste);
	addSeparator();

	QStringList tempList = PsiOptions::instance()->getOption("options.ui.chat.templates", QStringList()).toStringList();
	for (int i = 0; i < tempList.size(); i++) {
		QString str1 = tempList.at(i);
		if (!str1.isEmpty()) {
			QString str2;
			if (str1.length() > 30) {
				str2 = str1.left(30).replace("&", "&&") + "...";
			} else {
				str2 = str1.replace("&", "&&");
			}
			QAction *act = new QAction(str2, this);
			addAction(act);
			connect(act, SIGNAL(triggered()), mapper, SLOT(map()));
			mapper->setMapping(act, str1);
		}
	}
	connect(mapper, SIGNAL(mapped(const QString &)), SIGNAL(doTemplateText(const QString &)));
}

void SendButtonTemplatesMenu::clickOnlyPaste()
{
	if (PsiOptions::instance()->getOption("options.ui.chat.only-paste-template").toBool())
		PsiOptions::instance()->setOption("options.ui.chat.only-paste-template", false);
	else
		PsiOptions::instance()->setOption("options.ui.chat.only-paste-template", true);
}

void SendButtonTemplatesMenu::optionChanged(const QString& option)
{
	if (option == "options.ui.chat.disable-paste-send") {
		updatePsAction();
	} else if (option == "options.ui.chat.css") {
		setStyleSheet(PsiOptions::instance()->getOption("options.ui.chat.css").toString());
	//} else if (option == "options.ui.chat.only-paste-template") {
	//	onlyPaste->setChecked(PsiOptions::instance()->getOption("options.ui.chat.only-paste-template").toBool());
	}
}
